@@ -6,7 +6,7 @@ tfr.predict <- function(mcmc.set=NULL, end.year=2100,
 						use.tfr3=TRUE, burnin3=2000,
 						mu=2.1, rho=0.8859, sigmaAR1=0.1016,
 						min.tfr=0.5, use.correlation=FALSE,
-						save.as.ascii=0, output.dir = NULL,
+						save.as.ascii=0, output.dir = NULL, subdir = "predictions", 
 						low.memory=TRUE,
 						seed=NULL, verbose=TRUE, uncertainty=FALSE, ...) {
 	if(!is.null(mcmc.set)) {
@@ -55,7 +55,7 @@ tfr.predict <- function(mcmc.set=NULL, end.year=2100,
 					start.year=start.year, nr.traj=nr.traj, burnin=burnin, thin=thin, use.tfr3=has.phase3, burnin3=burnin3,
 					mu=mu, rho=rho,  sigmaAR1 = sigmaAR1, min.tfr=min.tfr, use.correlation=use.correlation,
 					save.as.ascii=save.as.ascii,
-					output.dir=output.dir, verbose=verbose, uncertainty=uncertainty, ...))			
+					output.dir=output.dir, subdir = subdir, verbose=verbose, uncertainty=uncertainty, ...))			
 }
 
 .find.burnin.nr.traj.from.diag <- function(diag.list, verbose = FALSE) {
@@ -88,7 +88,8 @@ get.burnin.nrtraj.from.diagnostics <- function(sim.dir, ...) {
 }
 
 tfr.predict.extra <- function(sim.dir=file.path(getwd(), 'bayesTFR.output'), 
-					prediction.dir=sim.dir, countries = NULL, save.as.ascii=0, verbose=TRUE, uncertainty=FALSE,
+					prediction.dir=sim.dir, subdir = "predictions", countries = NULL, save.as.ascii=0, 
+					verbose=TRUE, uncertainty=FALSE,
 					all.countries.required = TRUE, use.correlation = NULL) {
 	# Run prediction for given countries/regions (as codes). If they are not given it will be set to countries 
 	# for which there are MCMC results but no prediction.
@@ -96,9 +97,9 @@ tfr.predict.extra <- function(sim.dir=file.path(getwd(), 'bayesTFR.output'),
     mcmc.set <- get.tfr.mcmc(sim.dir)
 	if(is.null(mcmc.set))
 		stop('Error in "sim.dir" argument.')
-	pred <- get.tfr.prediction(sim.dir=prediction.dir)
+	pred <- get.tfr.prediction(sim.dir=prediction.dir, subdir = subdir)
 	if(is.null(pred))
-		stop('Error in "prediction.dir" argument.')
+		stop('Error in "sim.dir", "prediction.dir" or/and "subdir" argument. Use available.tfr.predictions() to check on valid predictions directories.')
 	if(length(setdiff(pred$mcmc.set$meta$regions$country_code, mcmc.set$meta$regions$country_code)) > 0)
 		stop('Prediction is inconsistent with the mcmc results. Use tfr.predict.')
 	if(is.null(countries)) {
@@ -125,7 +126,7 @@ tfr.predict.extra <- function(sim.dir=file.path(getwd(), 'bayesTFR.output'),
 									use.correlation=use.correlation, 
 									mu=pred$mu, rho=pred$rho, sigmaAR1=pred$sigmaAR1, 
 									min.tfr=pred$min.tfr, countries=countries.idx, save.as.ascii=0, output.dir=prediction.dir,
-									force.creating.thinned.mcmc=TRUE,
+									subdir = subdir, force.creating.thinned.mcmc=TRUE,
 									write.summary.files=FALSE, write.trajectories=TRUE, verbose=verbose, 
 									uncertainty=uncertainty, all.countries.required = all.countries.required)
 									
@@ -173,7 +174,8 @@ make.tfr.prediction <- function(mcmc.set, start.year=NULL, end.year=2100, replac
 								use.correlation=FALSE, countries = NULL,
 								adj.factor1=NA, adj.factor2=0, forceAR1=FALSE,
 								boost.first.period.in.phase2=TRUE,
-							    save.as.ascii=0, output.dir = NULL, write.summary.files=TRUE, 
+							    save.as.ascii=0, output.dir = NULL, subdir = "predictions", 
+								write.summary.files=TRUE, 
 							    is.mcmc.set.thinned=FALSE, force.creating.thinned.mcmc=FALSE,
 							    write.trajectories=TRUE, 
 							    verbose=verbose, uncertainty=FALSE, all.countries.required = TRUE){
@@ -213,10 +215,10 @@ make.tfr.prediction <- function(mcmc.set, start.year=NULL, end.year=2100, replac
 	
 	#setup output directory
 	if (is.null(output.dir)) output.dir <- meta$output.dir
-	outdir <- file.path(output.dir, 'predictions')
+	outdir <- file.path(output.dir, basename(subdir))
 	
 	if(is.null(countries)) {
-		if(!replace.output && has.tfr.prediction(sim.dir=output.dir))
+		if(!replace.output && has.tfr.prediction(sim.dir=output.dir, subdir = subdir))
 			stop('Prediction in ', outdir,
 			' already exists.\nSet replace.output=TRUE if you want to overwrite existing projections.')
 		unlink(outdir, recursive=TRUE)
@@ -392,10 +394,13 @@ make.tfr.prediction <- function(mcmc.set, start.year=NULL, end.year=2100, replac
 	  }
 	  else
 	  {
-	    for(year in 1:fps.end.obs.index) 
+	      last.not.na <- 1
+	    for(year in 1:fps.end.obs.index) {
 	      all.f_ps[icountry,year,] <- all.tfr.list[[prediction.countries[icountry]]][all.T_end.min+year-1]
+	      if(!is.na(all.f_ps[icountry,year,1])) last.not.na <- year
+	    }
 	    if (!is.null(mcmc.set$meta$ar.phase2) && (mcmc.set$meta$ar.phase2))
-	      f_ps_previous[, icountry] <- all.tfr.list[[prediction.countries[icountry]]][all.T_end.min+year-2]
+	      f_ps_previous[, icountry] <- all.tfr.list[[prediction.countries[icountry]]][all.T_end.min+last.not.na-2]
 	  }
 		first.two.na <- which(is.na(all.f_ps[icountry,,1]))[1:2]
 		which.Wsecond[icountry] <- first.two.na[2]
@@ -1002,14 +1007,14 @@ do.convert.trajectories <- function(pred, n, output.dir, countries=NULL, verbose
 
 
 convert.tfr.trajectories <- function(dir=file.path(getwd(), 'bayesTFR.output'), 
-								 n=1000, output.dir=NULL, 
+								 n=1000, subdir = "predictions", output.dir=NULL, 
 								 verbose=FALSE) {
 	# Converts all trajectory rda files into UN ascii, selecting n trajectories by equal spacing.
 	if(n <= 0) return()
-	pred <- get.tfr.prediction(sim.dir=dir)
+	pred <- get.tfr.prediction(sim.dir=dir, subdir = subdir)
 	if (is.null(output.dir)) output.dir <- pred$output.directory
 	if(!file.exists(output.dir)) dir.create(output.dir, recursive=TRUE)
-	cat('Converting trajectories from', dir, '\n')
+	cat('Converting trajectories from', file.path(dir, subdir), '\n')
 	if (is.null(pred$na.index)) {
 		if(verbose) cat('Finding NA values in each country ...\n')
 		for (country in 1:pred$mcmc.set$meta$nr_countries) {
@@ -1025,11 +1030,11 @@ convert.tfr.trajectories <- function(dir=file.path(getwd(), 'bayesTFR.output'),
 }
 
 write.projection.summary <- function(dir=file.path(getwd(), 'bayesTFR.output'), 
-									 output.dir=NULL, revision=NULL, adjusted=FALSE, 
+                                     subdir = "predictions", output.dir=NULL, revision=NULL, adjusted=FALSE, 
 									 est.uncertainty = FALSE, ...) {
 # Writes three prediction summary files, one in a user-friendly format, one in a UN-format,
 # and one parameter file.
-	pred <- get.tfr.prediction(sim.dir=dir)
+	pred <- get.tfr.prediction(sim.dir=dir, subdir = subdir)
 	if (is.null(output.dir)) output.dir <- pred$output.directory
 	if(!file.exists(output.dir)) dir.create(output.dir, recursive=TRUE)
 	tfr.write.projection.summary.and.parameters(pred, output.dir, revision=revision, adjusted=adjusted, 
@@ -1115,12 +1120,12 @@ get.projection.summary.header.bayesTFR.prediction <- function(pred, ...)
 
 "get.UN.variant.names" <- function(pred, ...) UseMethod("get.UN.variant.names")
 get.UN.variant.names.bayesTFR.prediction <- function(pred, ...) 
-	return(c('BHM median', 'BHM80 lower',  'BHM80 upper', 'BHM95 lower',  'BHM95 upper', 'Low', 
+	return(c('BHM median', 'BHM80 lower',  'BHM80 upper', 'BHM95 lower',  'BHM95 upper', 'BHM mean', 'Low', 
 					'High', 'Constant fertility'))
 					
 "get.friendly.variant.names" <- function(pred, ...) UseMethod("get.friendly.variant.names")
 get.friendly.variant.names.bayesTFR.prediction <- function(pred, ...)
-	return(c('median', 'lower 80', 'upper 80', 'lower 95', 'upper 95', '-0.5child', '+0.5child', 'constant'))
+	return(c('median', 'lower 80', 'upper 80', 'lower 95', 'upper 95', 'mean', '-0.5child', '+0.5child', 'constant'))
 
 get.wpp.revision.number <- function(pred) {
 	wpps <- c(2008, 2010, 2012, seq(2015, by = 2, length = 3), seq(2022, by = 2, length = 10))
@@ -1170,7 +1175,7 @@ do.write.projection.summary <- function(pred, output.dir, revision=NULL, indicat
 	UN.variant.names <- get.UN.variant.names(pred)
 	friendly.variant.names <- get.friendly.variant.names(pred)
 	# the code is dependent on the following order of the variants (it's presumed):
-	# median, lower 80, upper 80, lower 95, upper 95, -1/2child, +1/2 child, constant
+	# median, lower 80, upper 80, lower 95, upper 95, mean, -1/2child, +1/2 child, constant
 	nr.var <- length(UN.variant.names)
 	result1 <- result2 <- NULL
 	
@@ -1181,7 +1186,10 @@ do.write.projection.summary <- function(pred, output.dir, revision=NULL, indicat
 		if(est.uncertainty){ # add uncertainty
 		    est.object <- get.tfr.estimation(mcmc.list = pred$mcmc.set, country = country.obj$code, 
 		                                     probs = c(0.5, 0.1, 0.9, 0.025, 0.975), adjust = adjusted)
+		    est.object.mean <- get.tfr.estimation(mcmc.list = pred$mcmc.set, country = country.obj$code, 
+		                                     probs = "mean", adjust = adjusted)
 		    this.tfr.unc <- as.matrix(est.object$tfr_quantile)[1:ltfr, -ncol(est.object$tfr_quantile)]
+		    this.tfr.unc <- cbind(this.tfr.unc, mean = unlist(est.object.mean$tfr_quantile[1:ltfr, 1]))
 		    #this.tfr <- unlist(est.object$tfr_quantile[,1])[1:ltfr]
 		}
 		this.result1 <- cbind(
@@ -1191,15 +1199,18 @@ do.write.projection.summary <- function(pred, output.dir, revision=NULL, indicat
 		# prediction
 		median <- get.median.from.prediction(pred, country.obj$index, country.obj$code, adjusted=adjusted, 
 		                                     est.uncertainty = est.uncertainty)
+		mean <- get.mean.from.prediction(pred, country.obj$index, country.obj$code, adjusted=adjusted, 
+		                                     est.uncertainty = est.uncertainty)
 		proj.result <- rbind(median, 
 							   get.traj.quantiles(pred, country.obj$index, country.obj$code, pi=80, adjusted=adjusted, 
 							                      est.uncertainty = est.uncertainty),
 							   get.traj.quantiles(pred, country.obj$index, country.obj$code, pi=95, adjusted=adjusted, 
-							                      est.uncertainty = est.uncertainty))
+							                      est.uncertainty = est.uncertainty),
+							   mean)
 		if(any(friendly.variant.names == '-0.5child'))
 			proj.result <- rbind(proj.result,
 					   get.half.child.variant(median))
-		#browser()
+
 		proj.result <- round(rbind(proj.result,
 							   		rep(median[1], nr.proj)), precision) # constant variant
 		if(est.uncertainty){ # user-friendly output contains observed years as well
@@ -1216,7 +1227,6 @@ do.write.projection.summary <- function(pred, output.dir, revision=NULL, indicat
 		result1 <- rbind(result1, this.result1)
 		for(ivar in 1:nr.var) {
 		    this.obs.tfr <- if(est.uncertainty) obs.tfr[ivar,] else this.tfr
-		    #browser()
 			result2 <- rbind(result2, cbind(revision=rep(revision, nr.proj.all), 
 								   variant=rep(e$UN_variants[e$UN_variants$Vshort==UN.variant.names[ivar],'VarID'], nr.proj.all),
 								   country=rep(country.obj$code, nr.proj.all),
@@ -1294,19 +1304,19 @@ get.tfr.shift <- function(country.code, pred) {
 	return(pred)
 }
 
-tfr.median.reset <- function(sim.dir, countries = NULL) {
+tfr.median.reset <- function(sim.dir, countries = NULL, ...) {
     if(is.null(countries)) {
-        pred <- get.tfr.prediction(sim.dir)
+        pred <- get.tfr.prediction(sim.dir, ...)
         pred$median.shift <- NULL
         store.bayesTFR.prediction(pred)
         cat('\nMedians for all countries reset.\n')
     } else
-	    for(country in countries) pred <- tfr.median.shift(sim.dir, country, reset=TRUE)
+	    for(country in countries) pred <- tfr.median.shift(sim.dir, country, reset=TRUE, ...)
 	invisible(pred)
 }
 
-tfr.median.shift <- function(sim.dir, country, reset=FALSE, shift=0, from=NULL, to=NULL) {
-	pred <- get.tfr.prediction(sim.dir)
+tfr.median.shift <- function(sim.dir, country, reset=FALSE, shift=0, from=NULL, to=NULL, ...) {
+	pred <- get.tfr.prediction(sim.dir, ...)
 	pred <- .bdem.median.shift(pred, type='tfr', country=country, reset=reset, 
 				shift=shift, from=from, to=to)
 	store.bayesTFR.prediction(pred)
@@ -1346,16 +1356,17 @@ tfr.median.shift <- function(sim.dir, country, reset=FALSE, shift=0, from=NULL,
 	return(pred)
 }
 
-tfr.median.set <- function(sim.dir, country, values, years=NULL) {
-	pred <- get.tfr.prediction(sim.dir)
+tfr.median.set <- function(sim.dir, country, values, years=NULL, ...) {
+	pred <- get.tfr.prediction(sim.dir, ...)
 	pred <- .bdem.median.set(pred, 'tfr', country=country, values=values, years=years)
 	store.bayesTFR.prediction(pred)
 	invisible(pred)
 }
 
-tfr.median.adjust <- function(sim.dir, countries, factor1=2/3, factor2=1/3, forceAR1=FALSE) {
-	pred <- get.tfr.prediction(sim.dir)
-	if (is.null(pred)) stop('No valid prediction in ', sim.dir)
+tfr.median.adjust <- function(sim.dir, countries, factor1=2/3, factor2=1/3, forceAR1=FALSE, subdir = "predictions") {
+	pred <- get.tfr.prediction(sim.dir, subdir = subdir)
+	if (is.null(pred)) stop('Prediction not found in ', file.path(sim.dir, subdir), 
+	                        '. Check available.tfr.predictions() and use the subdir argument to set non-standard prediction subdirectory.')
 	mcmc.set <- pred$mcmc.set
 	if(is.null(countries)) {
 		cat('\nNo countries given. Nothing to be done.\n')
@@ -1374,7 +1385,7 @@ tfr.median.adjust <- function(sim.dir, countries, factor1=2/3, factor2=1/3, forc
 									use.tfr3=pred$use.tfr3, mcmc3.set=m3.set, burnin3=pred$burnin3,
 									mu=pred$mu, rho=pred$rho, sigmaAR1=pred$sigmaAR1, min.tfr=pred$min.tfr,
 									countries=countries.idx, adj.factor1=factor1, adj.factor2=factor2,
-									forceAR1=forceAR1, save.as.ascii=0, output.dir=sim.dir,
+									forceAR1=forceAR1, save.as.ascii=0, output.dir=sim.dir, subdir = subdir,
 									write.summary.files=FALSE, is.mcmc.set.thinned=TRUE, 
 									write.trajectories=FALSE, verbose=FALSE)
 	new.means <- new.pred$traj.mean.sd[,1,2:dim(new.pred$traj.mean.sd)[3]]
@@ -1382,7 +1393,7 @@ tfr.median.adjust <- function(sim.dir, countries, factor1=2/3, factor2=1/3, forc
 		tfr.median.set(sim.dir, countries[icountry], new.means[get.country.object(countries[icountry], mcmc.set$meta)$index,])
 	}
 	# reload adjusted prediction
-	invisible(get.tfr.prediction(sim.dir))
+	invisible(get.tfr.prediction(sim.dir, subdir = subdir))
 }
 
 tfr.correlation <- function(meta, cor.pred=NULL, low.coeffs=c(0.11, 0.26, 0.05, 0.09),
